from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.base_error_response import BaseErrorResponse
from ...models.challenge_response import ChallengeResponse
from ...models.list_snapshots_response import ListSnapshotsResponse
from ...models.validation_error_response import ValidationErrorResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    organization_name: str,
    *,
    network_name: Union[None, Unset, str] = UNSET,
    limit: Union[None, Unset, int] = 0,
    filter_session: Union[Unset, bool] = False,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_network_name: Union[None, Unset, str]
    if isinstance(network_name, Unset):
        json_network_name = UNSET
    else:
        json_network_name = network_name
    params["network_name"] = json_network_name

    json_limit: Union[None, Unset, int]
    if isinstance(limit, Unset):
        json_limit = UNSET
    else:
        json_limit = limit
    params["limit"] = json_limit

    params["filter_session"] = filter_session

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/{organization_name}/api/v1/snapshots/".format(
            organization_name=organization_name,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ListSnapshotsResponse.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 422:
        response_422 = ValidationErrorResponse.from_dict(response.json())

        return response_422
    if response.status_code == 401:
        response_401 = ChallengeResponse.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = BaseErrorResponse.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    network_name: Union[None, Unset, str] = UNSET,
    limit: Union[None, Unset, int] = 0,
    filter_session: Union[Unset, bool] = False,
) -> Response[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    """Lists all snapshots for a network. Optionally filters by session.

    Args:
        organization_name (str):
        network_name (Union[None, Unset, str]):
        limit (Union[None, Unset, int]):  Default: 0.
        filter_session (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[BaseErrorResponse, ChallengeResponse, ValidationErrorResponse, list['ListSnapshotsResponse']]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        network_name=network_name,
        limit=limit,
        filter_session=filter_session,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    network_name: Union[None, Unset, str] = UNSET,
    limit: Union[None, Unset, int] = 0,
    filter_session: Union[Unset, bool] = False,
) -> Optional[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    """Lists all snapshots for a network. Optionally filters by session.

    Args:
        organization_name (str):
        network_name (Union[None, Unset, str]):
        limit (Union[None, Unset, int]):  Default: 0.
        filter_session (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[BaseErrorResponse, ChallengeResponse, ValidationErrorResponse, list['ListSnapshotsResponse']]
    """

    return sync_detailed(
        organization_name=organization_name,
        client=client,
        network_name=network_name,
        limit=limit,
        filter_session=filter_session,
    ).parsed


async def asyncio_detailed(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    network_name: Union[None, Unset, str] = UNSET,
    limit: Union[None, Unset, int] = 0,
    filter_session: Union[Unset, bool] = False,
) -> Response[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    """Lists all snapshots for a network. Optionally filters by session.

    Args:
        organization_name (str):
        network_name (Union[None, Unset, str]):
        limit (Union[None, Unset, int]):  Default: 0.
        filter_session (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[BaseErrorResponse, ChallengeResponse, ValidationErrorResponse, list['ListSnapshotsResponse']]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        network_name=network_name,
        limit=limit,
        filter_session=filter_session,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    network_name: Union[None, Unset, str] = UNSET,
    limit: Union[None, Unset, int] = 0,
    filter_session: Union[Unset, bool] = False,
) -> Optional[
    Union[
        BaseErrorResponse,
        ChallengeResponse,
        ValidationErrorResponse,
        list["ListSnapshotsResponse"],
    ]
]:
    """Lists all snapshots for a network. Optionally filters by session.

    Args:
        organization_name (str):
        network_name (Union[None, Unset, str]):
        limit (Union[None, Unset, int]):  Default: 0.
        filter_session (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[BaseErrorResponse, ChallengeResponse, ValidationErrorResponse, list['ListSnapshotsResponse']]
    """

    return (
        await asyncio_detailed(
            organization_name=organization_name,
            client=client,
            network_name=network_name,
            limit=limit,
            filter_session=filter_session,
        )
    ).parsed
